@@ -1,4 +1,4 @@
-//@line 41 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 41 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 
 const PREF_APP_UPDATE_ENABLED             = "app.update.enabled";
 const PREF_APP_UPDATE_AUTO                = "app.update.auto";
@@ -16,6 +16,7 @@ const PREF_APP_UPDATE_LASTUPDATETIME_FMT  = "app.update.lastUpdateTime.%ID%";
 const PREF_APP_EXTENSIONS_VERSION         = "app.extensions.version";
 const PREF_GENERAL_USERAGENT_LOCALE       = "general.useragent.locale";
 const PREF_APP_UPDATE_INCOMPATIBLE_MODE   = "app.update.incompatible.mode";
+const PREF_UPDATE_NEVER_BRANCH            = "app.update.never.";
 
 const URI_UPDATE_PROMPT_DIALOG  = "chrome://mozapps/content/update/updates.xul";
 const URI_UPDATE_HISTORY_DIALOG = "chrome://mozapps/content/update/history.xul";
@@ -54,8 +55,8 @@ const STATE_FAILED          = "failed";
 // From updater/errors.h:
 const WRITE_ERROR = 7;
 
-const DOWNLOAD_CHUNK_SIZE           = 65536;
-const DOWNLOAD_BACKGROUND_INTERVAL  = 60;  // seconds
+const DOWNLOAD_CHUNK_SIZE           = 300000; // bytes
+const DOWNLOAD_BACKGROUND_INTERVAL  = 600;    // seconds
 const DOWNLOAD_FOREGROUND_INTERVAL  = 0;
 
 // Values for the PREF_APP_UPDATE_INCOMPATIBLE_MODE pref. See documentation in
@@ -848,7 +849,7 @@ function UpdateService() {
     LOG("UpdateService", "XPCOM ABI unknown: updates are not possible.");
   }
 
-//@line 899 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 900 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 
   // Start the update timer only after a profile has been selected so that the
   // appropriate values for the update check are read from the user's profile.  
@@ -1134,9 +1135,11 @@ UpdateService.prototype = {
     //
     // The outcome we follow is determined as follows:
     // 
+    // Note:  all Major updates require notification and confirmation
+    // 
     // Update Type      Mode      Incompatible    Outcome
-    // Major            0         Yes or No       Auto Install
-    // Major            1         No              Auto Install
+    // Major            0         Yes or No       Notify and Confirm
+    // Major            1         No              Notify and Confirm
     // Major            1         Yes             Notify and Confirm
     // Major            2         Yes or No       Notify and Confirm
     // Minor            0         Yes or No       Auto Install
@@ -1151,6 +1154,11 @@ UpdateService.prototype = {
     // If app.update.enabled is set to false, an update check is not performed
     // at all, and so none of the decision making above is entered into.
     //
+    if (update.type == "major") {
+      LOG("Checker", "_shouldPrompt: Prompting because it is a major update");
+      return true;
+    }
+
     update.QueryInterface(Components.interfaces.nsIPropertyBag);
     try {
       var licenseAccepted = update.getProperty("licenseAccepted") == "true";
@@ -1158,11 +1166,6 @@ UpdateService.prototype = {
     catch (e) {
       licenseAccepted = false;
     }
-    if (update.licenseURL && !licenseAccepted) {
-      LOG("Checker", "_shouldPrompt: Prompting because of an un-accepted " + 
-          "license");
-      return true;
-    }
     
     var updateEnabled = getPref("getBoolPref", PREF_APP_UPDATE_ENABLED, true);
     if (!updateEnabled) {
@@ -1185,18 +1188,45 @@ UpdateService.prototype = {
       LOG("Checker", "_shouldPrompt: Prompting if there are incompatibilities");
       return !isCompatible(update);
     case 2:
-      // Mode 2 is do not prompt only if there are no incompatibilities for 
-      // minor updates only
-      LOG("Checker", "_shouldPrompt: Prompting if major or there are " + 
-          "incompatibilities");
-      return update.type == "minor" ? !isCompatible(update) : true;
+      // Mode 2 is do not prompt only if there are no incompatibilities
+      LOG("Checker", "_shouldPrompt: Prompting if there are incompatibilities");
+      return !isCompatible(update);
     }
     // Mode 0 is do not prompt regardless of incompatibilities
     LOG("Checker", "_shouldPrompt: Not prompting the user - they choose to " +
         "ignore incompatibilities");
     return false;
   },
-  
+ 
+  /**
+   * Determine if we have all the strings we need to show the major update UI
+   */
+  haveStringsForMajorUpdate: function() {
+    var sbs = 
+        Components.classes["@mozilla.org/intl/stringbundle;1"].
+        getService(Components.interfaces.nsIStringBundleService);
+    var updateBundle = sbs.createBundle(URI_UPDATES_PROPERTIES);
+    
+    var stringNamesForMajorUpdate =
+        ["introType_minor_app", "introType_major_app_and_version",
+         "licenseContentNotFound", "updateMoreInfoContentNotFound",
+         "licenseContentDownloading", "updateMoreInfoContentDownloading",
+         "updatesfound_minor.title", "updatesfound_major.title",
+         "downloadButton_minor", "downloadButton_major", "neverButton"];
+
+    try {
+      for (var i = 0; i < stringNamesForMajorUpdate.length; ++i) {
+        if (!updateBundle.GetStringFromName(stringNamesForMajorUpdate[i]))
+          return false;
+      }
+      return true;
+    }
+    catch (ex) {
+      LOG("Checker", "haveStringsForMajorUpdate: missing a string: " + ex);
+      return false;
+    }
+  },
+ 
   /**
    * Determine which of the specified updates should be installed.
    * @param   updates
@@ -1221,9 +1251,21 @@ UpdateService.prototype = {
         minorUpdate = newestMinor = updates[i];
     }
 
-    // If there's a major update, always try and fetch that one first, 
-    // otherwise fall back to the newest minor update.
-    return majorUpdate || minorUpdate;
+    // XXX sspitzer hack for the MOZILLA_1_8_0_BRANCH
+    if (majorUpdate && !this.haveStringsForMajorUpdate()) {
+      LOG("Checker", "selectUpdate:  have a major update, but can't offer it because we are missing strings");
+      majorUpdate = null;
+    }
+
+    // IMPORTANT
+    // If there's a minor update, always try and fetch that one first, 
+    // otherwise use the newest major update.
+    // selectUpdate() only returns one update.
+    // if major were to trump minor, and we said "never" to the major
+    // we'd never get the minor update, since selectUpdate()
+    // would return the major update that the user said "never" to
+    // (shadowing the important minor update with security fixes)
+    return minorUpdate || majorUpdate;
   },
   
   /**
@@ -1246,6 +1288,23 @@ UpdateService.prototype = {
     if (!update)
       return;
     
+    // check if the user said "never" to this version
+    // this check is done here, and not in selectUpdate() so that
+    // the user can get an upgrade they said "never" to if they
+    // manually do "Check for Updates..."
+    // note, selectUpdate() only returns one update.
+    // but in selectUpdate(), minor updates trump major updates
+    // if major trumps minor, and we said "never" to the major
+    // we'd never see the minor update.
+    // 
+    // note, the never decision should only apply to major updates
+    // see bug #350636 for a scenario where this could potentially
+    // be an issue
+    var neverPrefName = PREF_UPDATE_NEVER_BRANCH + update.version;
+    var never = getPref("getBoolPref", neverPrefName, false);
+    if (never && update.type == "major")
+      return;
+
     if (this._shouldPrompt(update))
       showPromptIfNoIncompatibilities(update);
     else {
@@ -2486,7 +2545,7 @@ TimerManager.prototype = {
   }
 };
 
-//@line 2537 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2596 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 /**
  * UpdatePrompt
  * An object which can prompt the user with information about updates, request
@@ -2628,7 +2687,7 @@ UpdatePrompt.prototype = {
     return this;
   }
 };
-//@line 2679 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2738 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 
 var gModule = {
   registerSelf: function(componentManager, fileSpec, location, type) {
@@ -2680,13 +2739,13 @@ var gModule = {
                className  : "Update Checker",
                factory    : #1#(Checker)
              },
-//@line 2731 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2790 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     prompt:  { CID        : Components.ID("{27ABA825-35B5-4018-9FDD-F99250A0E722}"),
                contractID : "@mozilla.org/updates/update-prompt;1",
                className  : "Update Prompt",
                factory    : #1#(UpdatePrompt)
              },
-//@line 2737 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2796 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     timers:  { CID        : Components.ID("{B322A5C0-A419-484E-96BA-D7182163899F}"),
                contractID : "@mozilla.org/updates/timer-manager;1",
                className  : "Timer Manager",
@@ -2717,14 +2776,14 @@ function NSGetModule(compMgr, fileSpec) {
  *          the specified update, false otherwise.
  */
 function isCompatible(update) {
-//@line 2768 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2827 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   var em = 
       Components.classes["@mozilla.org/extensions/manager;1"].
       getService(Components.interfaces.nsIExtensionManager);
   var items = em.getIncompatibleItemList("", update.extensionVersion,
     nsIUpdateItem.TYPE_ADDON, false, { });
   return items.length == 0;
-//@line 2777 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2836 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
 }
 
 /**
@@ -2743,7 +2802,7 @@ function showPromptIfNoIncompatibilities(update) {
     prompter.showUpdateAvailable(update);
   }
 
-//@line 2796 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2855 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
   /**
    * Determines if an addon is compatible with a particular update.
    * @param   addon
@@ -2843,6 +2902,6 @@ function showPromptIfNoIncompatibilities(update) {
     em.update([], 0, mode != 0, listener);
   }
   else
-//@line 2896 "/usr/obj/ports/mozilla-thunderbird-1.5.0.7/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
+//@line 2955 "/usr/obj/ports/mozilla-thunderbird-1.5.0.8/mozilla/toolkit/mozapps/update/src/nsUpdateService.js.in"
     showPrompt(update);
 }
